/*------------------------------*/
/*  Intro a la programac de     */
/*  juegos, por Nacho Cabanes   */
/*                              */
/*   Nivel.cpp:                 */
/*     La pantalla del nivel 1  */
/*     Miniminer (version 0.10) */
/*     Fichero de desarrollo    */
/*                              */
/*  Comprobado con:             */
/*  - DevC++ 4.9.9.2(gcc 3.4.2) */
/*    y Allegro 4.03 - WinXP    */
/*  - gcc 4.4.3 y Allegro 4.2   */
/*    en Ubuntu 10.04           */
/*------------------------------*/

#include "Hardware.h"
#include "ElementoGraf.h"
#include "Nivel.h"

#ifdef __cplusplus
    #include <cstdlib>
    #include <cstdio>
#else
    #include <stdlib.h>
    #include <stdio.h>
#endif

#include <math.h>
#include <string>

#include <allegro.h>

Nivel::Nivel()
{
    nivelActual = 1;
    nivelMaximo = 2;

    anchoImagen = 16;
    altoImagen = 16;
    margenIzq = 64;
    margenSup = 48;

    int colum, fila;

    for(colum=0; colum<MAXCOLS; colum++)
        for (fila=0; fila<MAXFILAS; fila++)
            fragmentoNivel[fila][colum] = new ElementoGraf();

    leerDeFichero();
}


/** dibujarOculta: muestra el nivel sobre el fondo
 *  de pantalla, con su imagen correspondiente
 *  segun el tipo de ladrillo del que se trate
 */

void Nivel::dibujarOculta(Hardware h)
{
    int colum, fila;

    h.borrarOculta();

    for(colum=0; colum < MAXCOLS; colum++)
      for (fila=0; fila < MAXFILAS; fila++)
      {
        h.dibujarOculta( *fragmentoNivel[fila][colum] );
      }
}



/** leerDeFichero: lee de fichero los datos del nivel actual
 */
void Nivel::leerDeFichero()
{
    // Si hay fichero de datos, lo leo
    FILE *fichDatos;
    char linea[MAXCOLS+2];
    char nombreFich[50];
    sprintf(nombreFich, "nivel%03d.dat", nivelActual);
    imagenFondo = new ElementoGraf();
    imagenFondo->crearDesdeFichero("fondoVacio.bmp");
    fichDatos = fopen(nombreFich, "rt");
    llavesRestantes = 0;
    if (fichDatos != NULL)
        for (int fila=0; fila < MAXFILAS; fila++)   {
            fgets(linea,MAXCOLS+1,fichDatos);
            if (strlen(linea) < 5) // Salto los avances de linea de Windows
              fgets(linea,MAXCOLS+1,fichDatos);

            for (int colum=0; colum < MAXCOLS; colum++)   {
              mapa[fila][colum] = linea[colum];
              // Recoloco fragmentos
              fragmentoNivel[fila][colum] -> moverA(
                margenIzq + colum*anchoImagen, margenSup + fila*altoImagen);
              // Y leo su imagen desde fichero
              switch(linea[colum]) {
                  case 'S':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("suelo.bmp");
                       break;
                  case 'N':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("suelo2.bmp");
                       break;
                  case 'F':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("sueloFragil.bmp");
                       break;
                  case 'O':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("sueloFragil2.bmp");
                       break;
                  case 'L':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("ladrillo.bmp");
                       break;
                  case 'M':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("ladrillo2.bmp");
                       break;
                  case 'V':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("llave.bmp");
                       llavesRestantes ++;
                       break;
                  case 'P':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("puerta.bmp");
                       break;
                  case 'D':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("deslizante.bmp");
                       break;
                  case 'A':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("arbol.bmp");
                       break;
                  case 'T':
                       fragmentoNivel[fila][colum]->crearDesdeFichero("techo.bmp");
                       break;
                  default:
                       fragmentoNivel[fila][colum]->crearDesdeFichero("fondoVacio.bmp");
                       break;
              }

            }

        }
        fclose(fichDatos);
}


bool Nivel::esPisableCasilla(int y, int x)
{
     if (mapa[y][x] == ' ')  // Si es espacio
        return true;

     if (mapa[y][x] == 'V')  // Si es llave
        return true;

     if (mapa[y][x] == 'P')  // Si es puerta
        return true;

     return false;  // Resto de casos: no pisable
}


bool Nivel::esPosibleMover(int x1, int y1, int x2, int y2)
{
    // Coordenadas de las esquinas del personaje
    int xCasilla1 = (x1 - margenIzq) / anchoImagen;
    int yCasilla1 = (y1 - margenSup) / altoImagen;
    int xCasilla2 = (x2 - margenIzq) / anchoImagen;
    int yCasilla2 = (y2 - margenSup) / altoImagen;
    // Tambien zona a media altura, porque el personaje es mucho
    // mas alto que las casillas del fondo
    int yCasilla3 = (yCasilla1 + yCasilla2) / 2;

    // En vez de comprobar todas las casillas del fondo,
    // miro solo las que estan en las esquinas el personaje
    if ( ! esPisableCasilla( yCasilla1,xCasilla1 ) )
        return false;

    if ( ! esPisableCasilla( yCasilla1,xCasilla2 ) )
        return false;

    if ( ! esPisableCasilla( yCasilla2,xCasilla2 ) )
        return false;

    if ( ! esPisableCasilla( yCasilla2,xCasilla1 ) )
        return false;

    // Y las zonas a media altura, porque el personaje es alto
    if ( ! esPisableCasilla( yCasilla3,xCasilla2 ) )
        return false;

    if ( ! esPisableCasilla( yCasilla3,xCasilla1 ) )
        return false;

    return true;
}


int Nivel::obtenerPuntosPosicion(int x1, int y1, int x2, int y2)
{
    // "V" indica una llave en el mapa (10 puntos)
    // "P" indica una puerta en el mapa (50 puntos
    //     si se han recogido todas las llaves)
    // "T" indica un trozo de techo que nos mata
    //     (estalactita, -1 puntos)

    int xCasilla1 = (x1 - margenIzq) / anchoImagen;
    int yCasilla1 = (y1 - margenSup) / altoImagen;
    int xCasilla2 = (x2 - margenIzq) / anchoImagen;
    int yCasilla2 = (y2 - margenSup) / altoImagen;
    // Tambien zona a media altura, como en "esPosibleMover"
    int yCasilla3 = (yCasilla1 + yCasilla2) / 2;

    // Primero veo si choca con una estalactita
    // (puntuacion -1: perder vida)
    if ((mapa[yCasilla1][xCasilla1] == 'T')
       || (mapa[yCasilla1][xCasilla2] == 'T')
       || (mapa[yCasilla2][xCasilla2] == 'T')
       || (mapa[yCasilla2][xCasilla1] == 'T')
       || (mapa[yCasilla3][xCasilla2] == 'T')
       || (mapa[yCasilla3][xCasilla1] == 'T'))
        return -1;

    // Si toca la puerta y no quedan llaves, 50 puntos
    if ( ((mapa[yCasilla1][xCasilla1] == 'P')
       || (mapa[yCasilla1][xCasilla2] == 'P')
       || (mapa[yCasilla2][xCasilla2] == 'P')
       || (mapa[yCasilla2][xCasilla1] == 'P')
       || (mapa[yCasilla3][xCasilla2] == 'P')
       || (mapa[yCasilla3][xCasilla1] == 'P'))
       && (llavesRestantes == 0) )
        return 50;

    // Despues veo si toca una llave
    // (puntuacion 10, y se borra la llave)
    if (mapa[yCasilla1][xCasilla1] == 'V')
    {
        fragmentoNivel[yCasilla1][xCasilla1]->tomarImagen(*imagenFondo);
        // No basta con
        // fragmentoNivel[yCasilla1][xCasilla2] = imagenFondo;
        // o cambiara su X y su Y
        mapa[yCasilla1][xCasilla1] = ' ';
        llavesRestantes --;
        return 10;
    }

    if (mapa[yCasilla1][xCasilla2] == 'V')
    {
        fragmentoNivel[yCasilla1][xCasilla2]->tomarImagen(*imagenFondo);
        mapa[yCasilla1][xCasilla2] = ' ';
        llavesRestantes --;
        return 10;
    }

    if (mapa[yCasilla2][xCasilla2] == 'V')
    {
        fragmentoNivel[yCasilla2][xCasilla2]->tomarImagen(*imagenFondo);
        mapa[yCasilla2][xCasilla2] = ' ';
        llavesRestantes --;
        return 10;
    }

    if (mapa[yCasilla2][xCasilla1] == 'V')
    {
        fragmentoNivel[yCasilla2][xCasilla1]->tomarImagen(*imagenFondo);
        mapa[yCasilla2][xCasilla1] = ' ';
        llavesRestantes --;
        return 10;
    }

    if (mapa[yCasilla3][xCasilla2] == 'V')
    {
        fragmentoNivel[yCasilla3][xCasilla2]->tomarImagen(*imagenFondo);
        mapa[yCasilla3][xCasilla2] = ' ';
        llavesRestantes --;
        return 10;
    }

    if (mapa[yCasilla3][xCasilla1] == 'V')
    {
        fragmentoNivel[yCasilla3][xCasilla1]->tomarImagen(*imagenFondo);
        mapa[yCasilla3][xCasilla1] = ' ';
        llavesRestantes --;
        return 10;
    }

    // Si no ha pasado nada de lo anterior, no hay puntos que obtener
    return 0;
}


/** avanzarSiguiente: Pasa al siguiente nivel
 */
void Nivel::avanzarSiguiente()
{
    nivelActual ++;
    if (nivelActual > nivelMaximo)
      nivelActual = 1;

    leerDeFichero();
}


/** getNumero: Devuelve el numero de nivel (para ser usado en el marcador)
 */
int Nivel::getNumero()
{
    return nivelActual;
}
