/*------------------------------*/
/*  Intro a la programac de     */
/*  juegos, por Nacho Cabanes   */
/*                              */
/*   Personaje.cpp:             */
/*     El "personaje" de        */
/*     Miniminer (version 0.07) */
/*     Fichero de desarrollo    */
/*                              */
/*  Comprobado con:             */
/*  - DevC++ 4.9.9.2(gcc 3.4.2) */
/*    y Allegro 4.03 - WinXP    */
/*  - gcc 4.4.3 y Allegro 4.2   */
/*    en Ubuntu Linux 10.04     */
/*------------------------------*/

#include "Hardware.h"
#include "ElementoGraf.h"
#include "Personaje.h"

#ifdef __cplusplus
#include <cstdlib>
#include <cstdio>
#include <time.h>
#else
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#endif
using namespace std;


/** constructor
 *  y lee los records
 */
Personaje::Personaje() {
    xInicial = 199;
    yInicial = 223;
    reiniciarPosicion();
    anchura = 24;
    altura = 32;
    crearDesdeFichero( "personaje.bmp" );
    reiniciarVidas();
    saltando = false;
    cayendo = false;
    desplazVertical = 2;
    desplazHorizontal = 4;
    incrXSalto = 0;
    fotogramaMvto = 0;

    pasosSaltoArriba[0] = -6; pasosSaltoArriba[1] = -6;
    pasosSaltoArriba[2] = -6; pasosSaltoArriba[3] = -6;
    pasosSaltoArriba[4] = -6; pasosSaltoArriba[5] = -4;
    pasosSaltoArriba[6] = -4; pasosSaltoArriba[7] = -2;
    pasosSaltoArriba[8] = -2; pasosSaltoArriba[9] = 0;
    pasosSaltoArriba[10] = 0; pasosSaltoArriba[11] = 2;
    pasosSaltoArriba[12] = 2; pasosSaltoArriba[13] = 4;
    pasosSaltoArriba[14] = 4; pasosSaltoArriba[15] = 4;
    pasosSaltoArriba[16] = 6; pasosSaltoArriba[17] = 6;
    pasosSaltoArriba[18] = 6; pasosSaltoArriba[19] = 6;

}



/** moverDerecha: mueve el personaje hacia la derecha
 */
void Personaje::moverDerecha(Nivel n) {
    if (saltando || cayendo)
       return;
    if (n.esPosibleMover(posX+desplazHorizontal, posY+altura-4,
           posX+anchura+desplazHorizontal, posY+altura))
        posX += desplazHorizontal;
    cayendo = true;
}

/** moverIzquierda: mueve el personaje hacia la izquierda
 */
void Personaje::moverIzquierda(Nivel n) {
    if (saltando || cayendo)
       return;
    if (n.esPosibleMover(posX-desplazHorizontal, posY+altura-4,
            posX+anchura-desplazHorizontal, posY+altura))
        posX -= desplazHorizontal;
    cayendo = true;
}


/** saltar: comienza la secuencia de salto
 */
void Personaje::saltar() {
    if (saltando)
       return;
    saltando = true;
    fotogramaMvto = 0;
    incrXSalto = 0;
}

/** saltarDerecha: comienza la secuencia de salto hacia la derecha
 */
void Personaje::saltarDerecha() {
    saltar();
    incrXSalto = desplazHorizontal;
}


/** saltarIzquierda: comienza la secuencia de salto hacia la izquierda
 */
void Personaje::saltarIzquierda() {
    saltar();
    incrXSalto = -desplazHorizontal;
}

/** mover: mueve el personaje cuando se esta moviendo "solo"
    (durante un salto)
 */
void Personaje::mover(Nivel n) {
    // Tamao vertical: personaje + 20% aprox
    // Tamao horizontal: 4.5 casillas aprox
    if (saltando)
    {

        int xProxMov = posX + incrXSalto;
        int yProxMov = posY + pasosSaltoArriba[ fotogramaMvto ];
        bool subiendoSalto = (pasosSaltoArriba[ fotogramaMvto ] < 0);

        // Si todava se puede mover, avanzo
        if (n.esPosibleMover(xProxMov, yProxMov+altura-4,
           xProxMov+anchura, yProxMov+altura)
           || subiendoSalto )
        {
           posX = xProxMov;
           posY = yProxMov;
        }
        // Y si no, quiz est cayendo
        else
        {
            saltando = false;
            cayendo = true;
        }

        fotogramaMvto ++;
        if (fotogramaMvto >= MVTOS_SALTO)
        {
           saltando = false;
           cayendo = true;
        }
    }
    else if (cayendo)
    {
        if (n.esPosibleMover(posX, posY+desplazVertical+altura-4,
           posX+anchura, posY+desplazVertical+altura))
        {
           posY += desplazVertical;
        }
        else
            cayendo = false;
    }
}

/** reiniciarPosicion: devuelve el personaje a sus coordenadas iniciales
 */
void Personaje::reiniciarPosicion() {
    posX = xInicial;
    posY = yInicial;
    saltando = false;
}


int Personaje::getVidas() {
    return numVidas;
}


void Personaje::disminuirVidas() {
    numVidas --;
}


void Personaje::reiniciarVidas() {
    numVidas = 3;
}
